// Web Safe Color Cube - a CBE Application
// Mike Foster, mfoster.com, cross-browser.com


var cubeContainer, control, options, octantIndex, chip;
  
function windowOnload() {
  //
  options = new Object();
  octantIndex = new Array(0,3,18,21,108,111,126,129);
  //
  for (var i=0; i<8; ++i) {
    iterateCube(i, initOctantsIterator);
  }
  //
  chip = new Array();
  chip[0] = cbeGetElementById('chip0').cbe;
  chip[1] = cbeGetElementById('chip1').cbe;
  chip[2] = cbeGetElementById('chip2').cbe;
  control = cbeGetElementById('control').cbe;
  cubeContainer = cbeGetElementById('cubeContainer').cbe;
  //
  init();
  //
  document.cbe.addEventListener('mousemove', mousemoveListener);
  document.cbe.addEventListener('click', clickListener);
}

function init() {
  options.x = 'r';
  options.y = 'g';
  options.z = 'b';
  options.nums = false;
  options.size = 24;
  options.margin = 2 + (options.size / 2);
  options.inc = 0x33;
  //
  var f = document.forms.f1;
  f.status.value = 'Initializing...';
  f.rgb[0].checked = true;
  f.nums.checked = false;
  f.size.value = options.size;
  f.view[0].checked = true;
  //
  with (control) {
    resizeTo(150, document.cbe.height());
    moveTo(0,0);
    show();
  }
  with (chip[0]) {
    background('#0000FF');
    resizeTo(document.cbe.width()-control.width(), 50);
    moveTo(control.width(),0);
    show();
  }
  with (chip[1]) {
    background('#0000CC');
    resizeTo(parentNode.width()/2-20, parentNode.height()/2);
    moveTo('w',20);
    show();
  }
  with (chip[2]) {
    background('#000099');
    resizeTo(parentNode.width()/2-20, parentNode.height()/2);
    moveTo('e',20);
    show();
  }
  with (cubeContainer) {
    resizeTo(document.cbe.width()-control.width(), document.cbe.height()-chip[0].height());
    moveTo(control.width(), chip[0].height());
    show();
  }
  //
  paintCube('r', 'g', 'b', 1);
}

function updateChipColors() {
  var i, s = document.forms.f1.cColor.value;
  var c, a = s.split('\n');
  for (i=0; i<3 && i<a.length-1; ++i) {
    c = a[i].toUpperCase();
    if (c.charAt(0) == '#') {
      chip[i].background(c.substr(0, 7));
    }
    else if (c.indexOf('RGB') != -1) {
      chip[i].background(c);
    }
  }
}

function updateSize(sz) {
  var f = document.forms.f1;
  f.rgb[0].checked = true;
  f.nums.checked = false;
  f.view[0].checked = true;
  options.size = sz;
  options.margin = 2 + (options.size / 2);
  paintCube(0,0,0,1);
}

function initOctantsIterator(octNum, swatchIndex, cbe, r,g,b) {
  cbe.octNum = octNum;
}

function viewCube() {
  document.forms.f1.status.value = 'Positioning...';
  setTimeout("iterateCube(-1, viewCubeIterator)", 125);
}

function viewCubeIterator(octNum, swatchIndex, cbe) {
  cbe.moveTo(cbe.swatchLeft, cbe.swatchTop);
  if (swatchIndex == 215) document.forms.f1.status.value = '';
}

function viewOctants() {
  document.forms.f1.status.value = 'Positioning...';
  setTimeout("_viewOctants()", 125);
}
function _viewOctants() {
  var ix,iy,w,h, unit = 4*options.size + 3*options.margin;
  w = 16*options.size + 11*options.margin;
  ix = (cubeContainer.width() - w) / 2;
  h = 2*unit + unit/2;
  iy =  (cubeContainer.height() - h) / 2;
  octMoveTo(0, ix, iy);
  octMoveTo(1, ix+2*unit, iy);
  octMoveTo(2, ix+0, iy+unit);
  octMoveTo(3, ix+2*unit, iy+unit);
  octMoveTo(4, ix+unit, iy+unit/2);
  octMoveTo(5, ix+3*unit, iy+unit/2);
  octMoveTo(6, ix+unit, iy+unit+unit/2);
  octMoveTo(7, ix+3*unit, iy+unit+unit/2);
  document.forms.f1.status.value = '';
}

function viewPlanes() {
  document.forms.f1.status.value = 'Positioning...';
  setTimeout("iterateCube(-1, viewPlanesIterator)", 125);
}
function viewPlanesIterator(octNum, swatchIndex, cbe, xOfs, yOfs) {
  var unit = 6*(options.size + options.margin);
  xOfs += 10;
  yOfs += 10;
  if (swatchIndex < 36) {
    cbe.moveTo(xOfs, yOfs);
  }
  else if (swatchIndex < 72) {
    cbe.moveTo(unit+xOfs, yOfs);
  }
  else if (swatchIndex < 108) {
    cbe.moveTo(2*unit+xOfs, yOfs);
  }
  else if (swatchIndex < 144) {
    cbe.moveTo(xOfs, unit+yOfs);
  }
  else if (swatchIndex < 180) {
    cbe.moveTo(unit+xOfs, unit+yOfs);
  }
  else if (swatchIndex < 216) {
    cbe.moveTo(2*unit+xOfs, unit+yOfs);
  }
  if (swatchIndex == 215) document.forms.f1.status.value = '';
}

function octMoveTo(qn,qx,qy) {
  iterateCube(qn, octMoveToIterator, qx, qy);
}

function octMoveToIterator(octNum, swatchIndex, cbe, xOfs, yOfs, r,g,b, qx, qy) {
  cbe.moveTo(qx + xOfs, qy + yOfs);
}

function clickListener(e) {
  if (e.cbeTarget.id.indexOf('swatch') != -1 || e.cbeTarget.id.indexOf('chip') != -1) {
    var b = e.cbeTarget.background().toUpperCase();
    var ta = document.forms.f1.cColor;
    ta.value = b + '\n' + ta.value;
    updateChipColors();
  }
}

function mousemoveListener(e) {
  var b = "";
  if (e.cbeTarget.id.indexOf('swatch') != -1 || e.cbeTarget.id.indexOf('chip') != -1) {
    b = e.cbeTarget.background().toUpperCase();
  }
  document.forms.f1.mmColor.value = b;
}

function paintCube(ccX, ccY, ccZ, init) {
  var s;
  if (init) {
    var ix,iy,sz;
    sz = 6*options.size + 2*5*options.margin;
    ix = (cubeContainer.width() - sz) / 2;
    iy =  (cubeContainer.height() - sz) / 2;
    s = 'Building...';
    setTimeout("iterateCube(-1,initCubeIterator,"+ix+","+iy+",'"+options.x+"','"+options.y+"','"+options.z+"');", 125);
  }  
  else {
    if (arguments.length == 3) {
      options.x = ccX;
      options.y = ccY;
      options.z = ccZ;
    }
    s = 'Painting...';
    setTimeout("iterateCube(-1,paintCubeIterator,'"+options.x+"','"+options.y+"','"+options.z+"')", 125);
  }
  document.forms.f1.status.value = s;
}

function paintCubeIterator(octNum, swatchIndex, cbe, xOfs, yOfs, r,g,b, ccX, ccY, ccZ) {
  var rgb = cbeHexString(eval(ccZ),2) + cbeHexString(eval(ccY),2) + cbeHexString(eval(ccX),2);
  with (cbe) {
    background('#' + rgb);
    innerHtml(options.nums ? rgb.toUpperCase() : " ");
  }
  if (swatchIndex == 215) document.forms.f1.status.value = '';
}

function initCubeIterator(octNum, swatchIndex, cbe, xOfs, yOfs, r,g,b, initialX, initialY, ccX, ccY, ccZ) {
  var rgb = cbeHexString(eval(ccZ),2) + cbeHexString(eval(ccY),2) + cbeHexString(eval(ccX),2);
  with (cbe) {
    resizeTo(options.size, options.size); 
    background('#' + rgb);
    moveTo(initialX + xOfs, initialY + yOfs);
    zIndex(swatchIndex);
    innerHtml(options.nums ? rgb.toUpperCase() : " ");
    show();
  }
  cbe.swatchLeft = initialX + xOfs;
  cbe.swatchTop = initialY + yOfs;
  cbe.swatchIndex = swatchIndex;
  cbe.isSwatch = true;
  if (swatchIndex == 215) document.forms.f1.status.value = '';
}

function createCube() {
  var i;
  for (i = 0; i < 6*6*6; ++i) {
    document.write("<div id='swatch" + i + "' class='swatch'></div>");
  }
}

function iterateCube(octNum, callBack, arg1, arg2, arg3, arg4, arg5, arg6) {
  var r,g,b,e, x,y,z, px=0,py=0, n, xOfs=0, yOfs=0, swatchesPerAxis;
  ox = oy = oz = 0;
  if (octNum == -1) {
    swatchesPerAxis = 6;
    n = 0;
  }
  else {
    swatchesPerAxis = 3;
    n = octantIndex[octNum];
  }
  for (z=0, r=0; z < swatchesPerAxis; ++z, r+=options.inc) {
    yOfs = py;
    for (y=0, g=0; y < swatchesPerAxis; ++y, g+=options.inc) {
      xOfs = px;
      for (x=0, b=0; x < swatchesPerAxis; ++x, b+=options.inc) {
        e = cbeGetElementById('swatch'+n).cbe;
        callBack(octNum, n, e, xOfs, yOfs, r,g,b, arg1, arg2, arg3, arg4, arg5, arg6);
        xOfs += options.size + options.margin;
        ++n;
      }
      if (octNum != -1) n += 3;
      yOfs += options.size + options.margin;
    }
    if (octNum != -1) n += 18;
    px += options.margin;
    py += options.margin;
  }
}



